// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.6
// 	protoc        v6.31.1
// source: intentService/intent.proto

package protobuf

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

var File_intentService_intent_proto protoreflect.FileDescriptor

const file_intentService_intent_proto_rawDesc = "" +
	"\n" +
	"\x1aintentService/intent.proto\x12\rintentServiceB?Z=github.com/NextHCI/second-brain-backends-go/protobuf;protobufb\x06proto3"

var file_intentService_intent_proto_goTypes = []any{}
var file_intentService_intent_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_intentService_intent_proto_init() }
func file_intentService_intent_proto_init() {
	if File_intentService_intent_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_intentService_intent_proto_rawDesc), len(file_intentService_intent_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   0,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_intentService_intent_proto_goTypes,
		DependencyIndexes: file_intentService_intent_proto_depIdxs,
	}.Build()
	File_intentService_intent_proto = out.File
	file_intentService_intent_proto_goTypes = nil
	file_intentService_intent_proto_depIdxs = nil
}
