# Code generated by parse_topics.py. DO NOT EDIT.

ENTRY_RECORD_PREPROCESS = "entry.record.preprocess"
ENTRY_RECORD_PREPROCESS_REPLY = "entry.record.preprocess.reply"
ENTRY_RECORD_PROCESS = "entry.record.process"
ENTRY_RECORD_UPDATE = "entry.record.update"
INTENT_RESPONSE = "intent.response"
INTENT_RECORD_GROUP = "intent.record.group"
CHAT_REQUEST = "chat.request"
CHAT_RESPONSE = "chat.response"
SCHEDULE_REQUEST = "schedule.request"
SCHEDULE_RESPONSE = "schedule.response"
FEEDBACK_REQUEST = "feedback.request"
FEEDBACK_RESPONSE = "feedback.response"
TAG_REQUEST = "tag.request"
TAG_RESPONSE = "tag.response"
