// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.6
// 	protoc        v6.31.1
// source: entryService/chat_message.proto

package protobuf

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type ChatMessage struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	ProtoId       string                 `protobuf:"bytes,1,opt,name=proto_id,json=protoId,proto3" json:"proto_id,omitempty"`
	Id            uint32                 `protobuf:"varint,2,opt,name=id,proto3" json:"id,omitempty"`
	Content       string                 `protobuf:"bytes,3,opt,name=content,proto3" json:"content,omitempty"`
	Role          string                 `protobuf:"bytes,4,opt,name=role,proto3" json:"role,omitempty"`
	SessionId     uint32                 `protobuf:"varint,5,opt,name=session_id,json=sessionId,proto3" json:"session_id,omitempty"`
	UserId        uint32                 `protobuf:"varint,6,opt,name=user_id,json=userId,proto3" json:"user_id,omitempty"`
	IsStream      bool                   `protobuf:"varint,7,opt,name=is_stream,json=isStream,proto3" json:"is_stream,omitempty"`
	IsStreamDone  bool                   `protobuf:"varint,8,opt,name=is_stream_done,json=isStreamDone,proto3" json:"is_stream_done,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ChatMessage) Reset() {
	*x = ChatMessage{}
	mi := &file_entryService_chat_message_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ChatMessage) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ChatMessage) ProtoMessage() {}

func (x *ChatMessage) ProtoReflect() protoreflect.Message {
	mi := &file_entryService_chat_message_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ChatMessage.ProtoReflect.Descriptor instead.
func (*ChatMessage) Descriptor() ([]byte, []int) {
	return file_entryService_chat_message_proto_rawDescGZIP(), []int{0}
}

func (x *ChatMessage) GetProtoId() string {
	if x != nil {
		return x.ProtoId
	}
	return ""
}

func (x *ChatMessage) GetId() uint32 {
	if x != nil {
		return x.Id
	}
	return 0
}

func (x *ChatMessage) GetContent() string {
	if x != nil {
		return x.Content
	}
	return ""
}

func (x *ChatMessage) GetRole() string {
	if x != nil {
		return x.Role
	}
	return ""
}

func (x *ChatMessage) GetSessionId() uint32 {
	if x != nil {
		return x.SessionId
	}
	return 0
}

func (x *ChatMessage) GetUserId() uint32 {
	if x != nil {
		return x.UserId
	}
	return 0
}

func (x *ChatMessage) GetIsStream() bool {
	if x != nil {
		return x.IsStream
	}
	return false
}

func (x *ChatMessage) GetIsStreamDone() bool {
	if x != nil {
		return x.IsStreamDone
	}
	return false
}

type ChatHistory struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	ProtoId       string                 `protobuf:"bytes,1,opt,name=proto_id,json=protoId,proto3" json:"proto_id,omitempty"`
	Messages      []*ChatMessage         `protobuf:"bytes,2,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ChatHistory) Reset() {
	*x = ChatHistory{}
	mi := &file_entryService_chat_message_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ChatHistory) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ChatHistory) ProtoMessage() {}

func (x *ChatHistory) ProtoReflect() protoreflect.Message {
	mi := &file_entryService_chat_message_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ChatHistory.ProtoReflect.Descriptor instead.
func (*ChatHistory) Descriptor() ([]byte, []int) {
	return file_entryService_chat_message_proto_rawDescGZIP(), []int{1}
}

func (x *ChatHistory) GetProtoId() string {
	if x != nil {
		return x.ProtoId
	}
	return ""
}

func (x *ChatHistory) GetMessages() []*ChatMessage {
	if x != nil {
		return x.Messages
	}
	return nil
}

var File_entryService_chat_message_proto protoreflect.FileDescriptor

const file_entryService_chat_message_proto_rawDesc = "" +
	"\n" +
	"\x1fentryService/chat_message.proto\x12\fentryService\"\xe1\x01\n" +
	"\vChatMessage\x12\x19\n" +
	"\bproto_id\x18\x01 \x01(\tR\aprotoId\x12\x0e\n" +
	"\x02id\x18\x02 \x01(\rR\x02id\x12\x18\n" +
	"\acontent\x18\x03 \x01(\tR\acontent\x12\x12\n" +
	"\x04role\x18\x04 \x01(\tR\x04role\x12\x1d\n" +
	"\n" +
	"session_id\x18\x05 \x01(\rR\tsessionId\x12\x17\n" +
	"\auser_id\x18\x06 \x01(\rR\x06userId\x12\x1b\n" +
	"\tis_stream\x18\a \x01(\bR\bisStream\x12$\n" +
	"\x0eis_stream_done\x18\b \x01(\bR\fisStreamDone\"_\n" +
	"\vChatHistory\x12\x19\n" +
	"\bproto_id\x18\x01 \x01(\tR\aprotoId\x125\n" +
	"\bmessages\x18\x02 \x03(\v2\x19.entryService.ChatMessageR\bmessagesB?Z=github.com/NextHCI/second-brain-backends-go/protobuf;protobufb\x06proto3"

var (
	file_entryService_chat_message_proto_rawDescOnce sync.Once
	file_entryService_chat_message_proto_rawDescData []byte
)

func file_entryService_chat_message_proto_rawDescGZIP() []byte {
	file_entryService_chat_message_proto_rawDescOnce.Do(func() {
		file_entryService_chat_message_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_entryService_chat_message_proto_rawDesc), len(file_entryService_chat_message_proto_rawDesc)))
	})
	return file_entryService_chat_message_proto_rawDescData
}

var file_entryService_chat_message_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_entryService_chat_message_proto_goTypes = []any{
	(*ChatMessage)(nil), // 0: entryService.ChatMessage
	(*ChatHistory)(nil), // 1: entryService.ChatHistory
}
var file_entryService_chat_message_proto_depIdxs = []int32{
	0, // 0: entryService.ChatHistory.messages:type_name -> entryService.ChatMessage
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_entryService_chat_message_proto_init() }
func file_entryService_chat_message_proto_init() {
	if File_entryService_chat_message_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_entryService_chat_message_proto_rawDesc), len(file_entryService_chat_message_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_entryService_chat_message_proto_goTypes,
		DependencyIndexes: file_entryService_chat_message_proto_depIdxs,
		MessageInfos:      file_entryService_chat_message_proto_msgTypes,
	}.Build()
	File_entryService_chat_message_proto = out.File
	file_entryService_chat_message_proto_goTypes = nil
	file_entryService_chat_message_proto_depIdxs = nil
}
