syntax = "proto3";

package entryService;

option go_package = "github.com/NextHCI/second-brain-backends-go/protobuf;protobuf";

message ChatMessage {
    string proto_id = 1;
    uint32 id = 2;
    string content = 3;
    string role = 4;
    uint32 session_id = 5;
    uint32 user_id = 6;
    bool is_stream = 7;
    bool is_stream_done = 8;
}

message ChatHistory {
    string proto_id = 1;
    repeated ChatMessage messages = 2;
}