syntax = "proto3";

package intentService;

option go_package = "github.com/NextHCI/second-brain-backends-go/protobuf;protobuf";

/*
enum IntentCategory {
  NOTE = 0;
  COMMAND = 1;
  SCHEDULE = 2;
  FEEDBACK = 3;
}

message Intent {
  string proto_id = 1;
  uint32 record_id = 2;
  uint32 user_id = 3;
  repeated IntentCategory categories = 4;
}
*/