syntax = "proto3";

package intentService;

option go_package = "github.com/NextHCI/second-brain-backends-go/protobuf;protobuf";

message RecordGroup {
  string proto_id = 1;
  uint32 user_id = 2;
  uint32 group_id = 3;
  string title = 4;
  string description = 5;
}