// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.6
// 	protoc        v6.31.1
// source: intentService/intent_response.proto

package protobuf

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type IntentResponseType int32

const (
	IntentResponseType_TEXT     IntentResponseType = 0
	IntentResponseType_MARKDOWN IntentResponseType = 1
	IntentResponseType_IMAGE    IntentResponseType = 2
	IntentResponseType_VIDEO    IntentResponseType = 3
	IntentResponseType_COMMAND  IntentResponseType = 4
	IntentResponseType_SCHEDULE IntentResponseType = 5
)

// Enum value maps for IntentResponseType.
var (
	IntentResponseType_name = map[int32]string{
		0: "TEXT",
		1: "MARKDOWN",
		2: "IMAGE",
		3: "VIDEO",
		4: "COMMAND",
		5: "SCHEDULE",
	}
	IntentResponseType_value = map[string]int32{
		"TEXT":     0,
		"MARKDOWN": 1,
		"IMAGE":    2,
		"VIDEO":    3,
		"COMMAND":  4,
		"SCHEDULE": 5,
	}
)

func (x IntentResponseType) Enum() *IntentResponseType {
	p := new(IntentResponseType)
	*p = x
	return p
}

func (x IntentResponseType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (IntentResponseType) Descriptor() protoreflect.EnumDescriptor {
	return file_intentService_intent_response_proto_enumTypes[0].Descriptor()
}

func (IntentResponseType) Type() protoreflect.EnumType {
	return &file_intentService_intent_response_proto_enumTypes[0]
}

func (x IntentResponseType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use IntentResponseType.Descriptor instead.
func (IntentResponseType) EnumDescriptor() ([]byte, []int) {
	return file_intentService_intent_response_proto_rawDescGZIP(), []int{0}
}

type IntentResponse struct {
	state             protoimpl.MessageState `protogen:"open.v1"`
	ProtoId           string                 `protobuf:"bytes,1,opt,name=proto_id,json=protoId,proto3" json:"proto_id,omitempty"`
	Type              string                 `protobuf:"bytes,2,opt,name=type,proto3" json:"type,omitempty"`
	UserId            uint32                 `protobuf:"varint,3,opt,name=user_id,json=userId,proto3" json:"user_id,omitempty"`
	RecordId          uint32                 `protobuf:"varint,4,opt,name=record_id,json=recordId,proto3" json:"record_id,omitempty"`
	GroupId           uint32                 `protobuf:"varint,5,opt,name=group_id,json=groupId,proto3" json:"group_id,omitempty"`
	Mode              uint32                 `protobuf:"varint,6,opt,name=mode,proto3" json:"mode,omitempty"`
	Index             int32                  `protobuf:"varint,7,opt,name=index,proto3" json:"index,omitempty"`
	Count             uint32                 `protobuf:"varint,8,opt,name=count,proto3" json:"count,omitempty"`
	IsEnd             bool                   `protobuf:"varint,9,opt,name=is_end,json=isEnd,proto3" json:"is_end,omitempty"`
	Content           string                 `protobuf:"bytes,10,opt,name=content,proto3" json:"content,omitempty"`
	FileId            string                 `protobuf:"bytes,11,opt,name=file_id,json=fileId,proto3" json:"file_id,omitempty"`
	ResponseMessageId string                 `protobuf:"bytes,12,opt,name=response_message_id,json=responseMessageId,proto3" json:"response_message_id,omitempty"`
	Status            string                 `protobuf:"bytes,13,opt,name=status,proto3" json:"status,omitempty"`
	unknownFields     protoimpl.UnknownFields
	sizeCache         protoimpl.SizeCache
}

func (x *IntentResponse) Reset() {
	*x = IntentResponse{}
	mi := &file_intentService_intent_response_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *IntentResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*IntentResponse) ProtoMessage() {}

func (x *IntentResponse) ProtoReflect() protoreflect.Message {
	mi := &file_intentService_intent_response_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use IntentResponse.ProtoReflect.Descriptor instead.
func (*IntentResponse) Descriptor() ([]byte, []int) {
	return file_intentService_intent_response_proto_rawDescGZIP(), []int{0}
}

func (x *IntentResponse) GetProtoId() string {
	if x != nil {
		return x.ProtoId
	}
	return ""
}

func (x *IntentResponse) GetType() string {
	if x != nil {
		return x.Type
	}
	return ""
}

func (x *IntentResponse) GetUserId() uint32 {
	if x != nil {
		return x.UserId
	}
	return 0
}

func (x *IntentResponse) GetRecordId() uint32 {
	if x != nil {
		return x.RecordId
	}
	return 0
}

func (x *IntentResponse) GetGroupId() uint32 {
	if x != nil {
		return x.GroupId
	}
	return 0
}

func (x *IntentResponse) GetMode() uint32 {
	if x != nil {
		return x.Mode
	}
	return 0
}

func (x *IntentResponse) GetIndex() int32 {
	if x != nil {
		return x.Index
	}
	return 0
}

func (x *IntentResponse) GetCount() uint32 {
	if x != nil {
		return x.Count
	}
	return 0
}

func (x *IntentResponse) GetIsEnd() bool {
	if x != nil {
		return x.IsEnd
	}
	return false
}

func (x *IntentResponse) GetContent() string {
	if x != nil {
		return x.Content
	}
	return ""
}

func (x *IntentResponse) GetFileId() string {
	if x != nil {
		return x.FileId
	}
	return ""
}

func (x *IntentResponse) GetResponseMessageId() string {
	if x != nil {
		return x.ResponseMessageId
	}
	return ""
}

func (x *IntentResponse) GetStatus() string {
	if x != nil {
		return x.Status
	}
	return ""
}

var File_intentService_intent_response_proto protoreflect.FileDescriptor

const file_intentService_intent_response_proto_rawDesc = "" +
	"\n" +
	"#intentService/intent_response.proto\x12\rintentService\"\xe2\x02\n" +
	"\x0eIntentResponse\x12\x19\n" +
	"\bproto_id\x18\x01 \x01(\tR\aprotoId\x12\x12\n" +
	"\x04type\x18\x02 \x01(\tR\x04type\x12\x17\n" +
	"\auser_id\x18\x03 \x01(\rR\x06userId\x12\x1b\n" +
	"\trecord_id\x18\x04 \x01(\rR\brecordId\x12\x19\n" +
	"\bgroup_id\x18\x05 \x01(\rR\agroupId\x12\x12\n" +
	"\x04mode\x18\x06 \x01(\rR\x04mode\x12\x14\n" +
	"\x05index\x18\a \x01(\x05R\x05index\x12\x14\n" +
	"\x05count\x18\b \x01(\rR\x05count\x12\x15\n" +
	"\x06is_end\x18\t \x01(\bR\x05isEnd\x12\x18\n" +
	"\acontent\x18\n" +
	" \x01(\tR\acontent\x12\x17\n" +
	"\afile_id\x18\v \x01(\tR\x06fileId\x12.\n" +
	"\x13response_message_id\x18\f \x01(\tR\x11responseMessageId\x12\x16\n" +
	"\x06status\x18\r \x01(\tR\x06status*]\n" +
	"\x12IntentResponseType\x12\b\n" +
	"\x04TEXT\x10\x00\x12\f\n" +
	"\bMARKDOWN\x10\x01\x12\t\n" +
	"\x05IMAGE\x10\x02\x12\t\n" +
	"\x05VIDEO\x10\x03\x12\v\n" +
	"\aCOMMAND\x10\x04\x12\f\n" +
	"\bSCHEDULE\x10\x05B?Z=github.com/NextHCI/second-brain-backends-go/protobuf;protobufb\x06proto3"

var (
	file_intentService_intent_response_proto_rawDescOnce sync.Once
	file_intentService_intent_response_proto_rawDescData []byte
)

func file_intentService_intent_response_proto_rawDescGZIP() []byte {
	file_intentService_intent_response_proto_rawDescOnce.Do(func() {
		file_intentService_intent_response_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_intentService_intent_response_proto_rawDesc), len(file_intentService_intent_response_proto_rawDesc)))
	})
	return file_intentService_intent_response_proto_rawDescData
}

var file_intentService_intent_response_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_intentService_intent_response_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_intentService_intent_response_proto_goTypes = []any{
	(IntentResponseType)(0), // 0: intentService.IntentResponseType
	(*IntentResponse)(nil),  // 1: intentService.IntentResponse
}
var file_intentService_intent_response_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_intentService_intent_response_proto_init() }
func file_intentService_intent_response_proto_init() {
	if File_intentService_intent_response_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_intentService_intent_response_proto_rawDesc), len(file_intentService_intent_response_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_intentService_intent_response_proto_goTypes,
		DependencyIndexes: file_intentService_intent_response_proto_depIdxs,
		EnumInfos:         file_intentService_intent_response_proto_enumTypes,
		MessageInfos:      file_intentService_intent_response_proto_msgTypes,
	}.Build()
	File_intentService_intent_response_proto = out.File
	file_intentService_intent_response_proto_goTypes = nil
	file_intentService_intent_response_proto_depIdxs = nil
}
