# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: entryService/record.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'entryService/record.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19\x65ntryService/record.proto\x12\x0c\x65ntryService\"\xbc\x02\n\x06Record\x12\x10\n\x08proto_id\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\r\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x11\n\x04text\x18\x04 \x01(\tH\x00\x88\x01\x01\x12\x0c\n\x04type\x18\x05 \x01(\t\x12\x0f\n\x07user_id\x18\x06 \x01(\r\x12\x10\n\x08group_id\x18\x07 \x01(\r\x12\x0f\n\x07\x66ile_id\x18\x08 \x01(\t\x12\x15\n\ris_recognized\x18\t \x01(\x08\x12\x12\n\nis_deleted\x18\n \x01(\x08\x12\x11\n\ttimestamp\x18\x0b \x01(\t\x12\x0c\n\x04role\x18\x0c \x01(\t\x12\x11\n\tis_stream\x18\r \x01(\x08\x12\x16\n\x0eis_stream_done\x18\x0e \x01(\x08\x12\x13\n\x0b\x66orce_reply\x18\x0f \x01(\x08\x12\x0c\n\x04tags\x18\x10 \x03(\t\x12\x0e\n\x06status\x18\x11 \x01(\tB\x07\n\x05_textB?Z=github.com/NextHCI/second-brain-backends-go/protobuf;protobufb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'entryService.record_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z=github.com/NextHCI/second-brain-backends-go/protobuf;protobuf'
  _globals['_RECORD']._serialized_start=44
  _globals['_RECORD']._serialized_end=360
# @@protoc_insertion_point(module_scope)
