.react-flow__node-base {
  background: #f8f8f8;
  border: 1px solid rgba(0, 0, 0, 50%);
  border-radius: 4px;
}

.react-flow__handle {
  position: relative;
  top: auto;
  left: auto;
  transform: none;
}

.handle {
  width: 8px;
  height: 8px;
  background: #224466;
}

.handle-horizontal {
  top: auto;
}

.handle-vertical {
  left: auto;
}

.label {
  max-width: 240px;
  padding: 16px 32px;
  font-size: 12px;
  text-align: center;
}

.label {
  flex-grow: 1;
}

.handles {
  position: absolute;
  display: flex;
  justify-content: space-around;
}

.handles-horizontal {
  width: 10px;
  height: 100%;
}

.handles-vertical {
  width: 100%;
  height: 10px;
}

.handles-horizontal.targets {
  top: 0px;
  left: -4px;
}

.handles-horizontal.sources {
  top: 0px;
  right: -2px;
}

.handles-vertical.targets {
  top: -4px;
  left: 0px;
}

.handles-vertical.sources {
  bottom: -2px;
  left: 0px;
}
