syntax = "proto3";

package entryService;

option go_package = "github.com/NextHCI/second-brain-backends-go/protobuf;protobuf";

message Record {
  string proto_id = 1;
  uint32 id = 2;
  string name = 3;
  optional string text = 4;
  string type = 5;
  uint32 user_id = 6;
  uint32 group_id = 7;
  string file_id = 8;
  bool is_recognized = 9;
  bool is_deleted = 10;
  string timestamp = 11;
  string role = 12;
  bool is_stream = 13;
  bool is_stream_done = 14;
  bool force_reply = 15;
  repeated string tags = 16;
  string status = 17;
} 
