// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.6
// 	protoc        v6.31.1
// source: scheduleService/schedule.proto

package protobuf

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Schedule struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	ProtoId       string                 `protobuf:"bytes,1,opt,name=proto_id,json=protoId,proto3" json:"proto_id,omitempty"`
	RecordId      uint32                 `protobuf:"varint,2,opt,name=record_id,json=recordId,proto3" json:"record_id,omitempty"`
	UserId        uint32                 `protobuf:"varint,3,opt,name=user_id,json=userId,proto3" json:"user_id,omitempty"`
	Result        string                 `protobuf:"bytes,4,opt,name=result,proto3" json:"result,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Schedule) Reset() {
	*x = Schedule{}
	mi := &file_scheduleService_schedule_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Schedule) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Schedule) ProtoMessage() {}

func (x *Schedule) ProtoReflect() protoreflect.Message {
	mi := &file_scheduleService_schedule_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Schedule.ProtoReflect.Descriptor instead.
func (*Schedule) Descriptor() ([]byte, []int) {
	return file_scheduleService_schedule_proto_rawDescGZIP(), []int{0}
}

func (x *Schedule) GetProtoId() string {
	if x != nil {
		return x.ProtoId
	}
	return ""
}

func (x *Schedule) GetRecordId() uint32 {
	if x != nil {
		return x.RecordId
	}
	return 0
}

func (x *Schedule) GetUserId() uint32 {
	if x != nil {
		return x.UserId
	}
	return 0
}

func (x *Schedule) GetResult() string {
	if x != nil {
		return x.Result
	}
	return ""
}

var File_scheduleService_schedule_proto protoreflect.FileDescriptor

const file_scheduleService_schedule_proto_rawDesc = "" +
	"\n" +
	"\x1escheduleService/schedule.proto\x12\x0fscheduleService\"s\n" +
	"\bSchedule\x12\x19\n" +
	"\bproto_id\x18\x01 \x01(\tR\aprotoId\x12\x1b\n" +
	"\trecord_id\x18\x02 \x01(\rR\brecordId\x12\x17\n" +
	"\auser_id\x18\x03 \x01(\rR\x06userId\x12\x16\n" +
	"\x06result\x18\x04 \x01(\tR\x06resultB?Z=github.com/NextHCI/second-brain-backends-go/protobuf;protobufb\x06proto3"

var (
	file_scheduleService_schedule_proto_rawDescOnce sync.Once
	file_scheduleService_schedule_proto_rawDescData []byte
)

func file_scheduleService_schedule_proto_rawDescGZIP() []byte {
	file_scheduleService_schedule_proto_rawDescOnce.Do(func() {
		file_scheduleService_schedule_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_scheduleService_schedule_proto_rawDesc), len(file_scheduleService_schedule_proto_rawDesc)))
	})
	return file_scheduleService_schedule_proto_rawDescData
}

var file_scheduleService_schedule_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_scheduleService_schedule_proto_goTypes = []any{
	(*Schedule)(nil), // 0: scheduleService.Schedule
}
var file_scheduleService_schedule_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_scheduleService_schedule_proto_init() }
func file_scheduleService_schedule_proto_init() {
	if File_scheduleService_schedule_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_scheduleService_schedule_proto_rawDesc), len(file_scheduleService_schedule_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_scheduleService_schedule_proto_goTypes,
		DependencyIndexes: file_scheduleService_schedule_proto_depIdxs,
		MessageInfos:      file_scheduleService_schedule_proto_msgTypes,
	}.Build()
	File_scheduleService_schedule_proto = out.File
	file_scheduleService_schedule_proto_goTypes = nil
	file_scheduleService_schedule_proto_depIdxs = nil
}
