syntax = "proto3";

package intentService;

option go_package = "github.com/NextHCI/second-brain-backends-go/protobuf;protobuf";

enum IntentResponseType {
	TEXT = 0;
	MARKDOWN = 1;
	IMAGE = 2;
	VIDEO = 3;
	COMMAND = 4;
  SCHEDULE = 5;
}

message IntentResponse {
  string proto_id = 1;
  string type = 2;
  uint32 user_id = 3;
  uint32 record_id = 4;
  uint32 group_id = 5;
  uint32 mode = 6;
  int32 index = 7;
  uint32 count = 8;
  bool is_end = 9;
  string content = 10;
  string file_id = 11;
  string response_message_id = 12;
  string status = 13;
}