syntax = "proto3";

package tagService;

option go_package = "github.com/NextHCI/second-brain-backends-go/protobuf;protobuf";

message Tag {
  string proto_id = 1;
  uint32 record_id = 2;
  uint32 user_id = 3;
  repeated string tags = 4;
}