import React from 'react';
import { createRoot } from 'react-dom/client';
import './index.css';

import { WorkFlowViewer } from './App.tsx';

createRoot(document.getElementById('root')!).render(
  <React.StrictMode>
    <WorkFlowViewer />
  </React.StrictMode>,
);
