// Code generated by parse_topics.py. DO NOT EDIT.
package protobuf

const ENTRY_RECORD_PREPROCESS = "entry.record.preprocess"
const ENTRY_RECORD_PREPROCESS_REPLY = "entry.record.preprocess.reply"
const ENTRY_RECORD_PROCESS = "entry.record.process"
const ENTRY_RECORD_UPDATE = "entry.record.update"
const INTENT_RESPONSE = "intent.response"
const INTENT_RECORD_GROUP = "intent.record.group"
const CHAT_REQUEST = "chat.request"
const CHAT_RESPONSE = "chat.response"
const SCHEDULE_REQUEST = "schedule.request"
const SCHEDULE_RESPONSE = "schedule.response"
const FEEDBACK_REQUEST = "feedback.request"
const FEEDBACK_RESPONSE = "feedback.response"
const TAG_REQUEST = "tag.request"
const TAG_RESPONSE = "tag.response"
