// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.6
// 	protoc        v6.31.1
// source: entryService/record.proto

package protobuf

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Record struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	ProtoId       string                 `protobuf:"bytes,1,opt,name=proto_id,json=protoId,proto3" json:"proto_id,omitempty"`
	Id            uint32                 `protobuf:"varint,2,opt,name=id,proto3" json:"id,omitempty"`
	Name          string                 `protobuf:"bytes,3,opt,name=name,proto3" json:"name,omitempty"`
	Text          *string                `protobuf:"bytes,4,opt,name=text,proto3,oneof" json:"text,omitempty"`
	Type          string                 `protobuf:"bytes,5,opt,name=type,proto3" json:"type,omitempty"`
	UserId        uint32                 `protobuf:"varint,6,opt,name=user_id,json=userId,proto3" json:"user_id,omitempty"`
	GroupId       uint32                 `protobuf:"varint,7,opt,name=group_id,json=groupId,proto3" json:"group_id,omitempty"`
	FileId        string                 `protobuf:"bytes,8,opt,name=file_id,json=fileId,proto3" json:"file_id,omitempty"`
	IsRecognized  bool                   `protobuf:"varint,9,opt,name=is_recognized,json=isRecognized,proto3" json:"is_recognized,omitempty"`
	IsDeleted     bool                   `protobuf:"varint,10,opt,name=is_deleted,json=isDeleted,proto3" json:"is_deleted,omitempty"`
	Timestamp     string                 `protobuf:"bytes,11,opt,name=timestamp,proto3" json:"timestamp,omitempty"`
	Role          string                 `protobuf:"bytes,12,opt,name=role,proto3" json:"role,omitempty"`
	IsStream      bool                   `protobuf:"varint,13,opt,name=is_stream,json=isStream,proto3" json:"is_stream,omitempty"`
	IsStreamDone  bool                   `protobuf:"varint,14,opt,name=is_stream_done,json=isStreamDone,proto3" json:"is_stream_done,omitempty"`
	ForceReply    bool                   `protobuf:"varint,15,opt,name=force_reply,json=forceReply,proto3" json:"force_reply,omitempty"`
	Tags          []string               `protobuf:"bytes,16,rep,name=tags,proto3" json:"tags,omitempty"`
	Status        string                 `protobuf:"bytes,17,opt,name=status,proto3" json:"status,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Record) Reset() {
	*x = Record{}
	mi := &file_entryService_record_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Record) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Record) ProtoMessage() {}

func (x *Record) ProtoReflect() protoreflect.Message {
	mi := &file_entryService_record_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Record.ProtoReflect.Descriptor instead.
func (*Record) Descriptor() ([]byte, []int) {
	return file_entryService_record_proto_rawDescGZIP(), []int{0}
}

func (x *Record) GetProtoId() string {
	if x != nil {
		return x.ProtoId
	}
	return ""
}

func (x *Record) GetId() uint32 {
	if x != nil {
		return x.Id
	}
	return 0
}

func (x *Record) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *Record) GetText() string {
	if x != nil && x.Text != nil {
		return *x.Text
	}
	return ""
}

func (x *Record) GetType() string {
	if x != nil {
		return x.Type
	}
	return ""
}

func (x *Record) GetUserId() uint32 {
	if x != nil {
		return x.UserId
	}
	return 0
}

func (x *Record) GetGroupId() uint32 {
	if x != nil {
		return x.GroupId
	}
	return 0
}

func (x *Record) GetFileId() string {
	if x != nil {
		return x.FileId
	}
	return ""
}

func (x *Record) GetIsRecognized() bool {
	if x != nil {
		return x.IsRecognized
	}
	return false
}

func (x *Record) GetIsDeleted() bool {
	if x != nil {
		return x.IsDeleted
	}
	return false
}

func (x *Record) GetTimestamp() string {
	if x != nil {
		return x.Timestamp
	}
	return ""
}

func (x *Record) GetRole() string {
	if x != nil {
		return x.Role
	}
	return ""
}

func (x *Record) GetIsStream() bool {
	if x != nil {
		return x.IsStream
	}
	return false
}

func (x *Record) GetIsStreamDone() bool {
	if x != nil {
		return x.IsStreamDone
	}
	return false
}

func (x *Record) GetForceReply() bool {
	if x != nil {
		return x.ForceReply
	}
	return false
}

func (x *Record) GetTags() []string {
	if x != nil {
		return x.Tags
	}
	return nil
}

func (x *Record) GetStatus() string {
	if x != nil {
		return x.Status
	}
	return ""
}

var File_entryService_record_proto protoreflect.FileDescriptor

const file_entryService_record_proto_rawDesc = "" +
	"\n" +
	"\x19entryService/record.proto\x12\fentryService\"\xd0\x03\n" +
	"\x06Record\x12\x19\n" +
	"\bproto_id\x18\x01 \x01(\tR\aprotoId\x12\x0e\n" +
	"\x02id\x18\x02 \x01(\rR\x02id\x12\x12\n" +
	"\x04name\x18\x03 \x01(\tR\x04name\x12\x17\n" +
	"\x04text\x18\x04 \x01(\tH\x00R\x04text\x88\x01\x01\x12\x12\n" +
	"\x04type\x18\x05 \x01(\tR\x04type\x12\x17\n" +
	"\auser_id\x18\x06 \x01(\rR\x06userId\x12\x19\n" +
	"\bgroup_id\x18\a \x01(\rR\agroupId\x12\x17\n" +
	"\afile_id\x18\b \x01(\tR\x06fileId\x12#\n" +
	"\ris_recognized\x18\t \x01(\bR\fisRecognized\x12\x1d\n" +
	"\n" +
	"is_deleted\x18\n" +
	" \x01(\bR\tisDeleted\x12\x1c\n" +
	"\ttimestamp\x18\v \x01(\tR\ttimestamp\x12\x12\n" +
	"\x04role\x18\f \x01(\tR\x04role\x12\x1b\n" +
	"\tis_stream\x18\r \x01(\bR\bisStream\x12$\n" +
	"\x0eis_stream_done\x18\x0e \x01(\bR\fisStreamDone\x12\x1f\n" +
	"\vforce_reply\x18\x0f \x01(\bR\n" +
	"forceReply\x12\x12\n" +
	"\x04tags\x18\x10 \x03(\tR\x04tags\x12\x16\n" +
	"\x06status\x18\x11 \x01(\tR\x06statusB\a\n" +
	"\x05_textB?Z=github.com/NextHCI/second-brain-backends-go/protobuf;protobufb\x06proto3"

var (
	file_entryService_record_proto_rawDescOnce sync.Once
	file_entryService_record_proto_rawDescData []byte
)

func file_entryService_record_proto_rawDescGZIP() []byte {
	file_entryService_record_proto_rawDescOnce.Do(func() {
		file_entryService_record_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_entryService_record_proto_rawDesc), len(file_entryService_record_proto_rawDesc)))
	})
	return file_entryService_record_proto_rawDescData
}

var file_entryService_record_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_entryService_record_proto_goTypes = []any{
	(*Record)(nil), // 0: entryService.Record
}
var file_entryService_record_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_entryService_record_proto_init() }
func file_entryService_record_proto_init() {
	if File_entryService_record_proto != nil {
		return
	}
	file_entryService_record_proto_msgTypes[0].OneofWrappers = []any{}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_entryService_record_proto_rawDesc), len(file_entryService_record_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_entryService_record_proto_goTypes,
		DependencyIndexes: file_entryService_record_proto_depIdxs,
		MessageInfos:      file_entryService_record_proto_msgTypes,
	}.Build()
	File_entryService_record_proto = out.File
	file_entryService_record_proto_goTypes = nil
	file_entryService_record_proto_depIdxs = nil
}
